#!/usr/bin/perl

## Date and Version
my $version= '1.0';
my $version_date = '30_nov_2011';

##################################################
## Documentation POD
##################################################

##################################################
## Modules
##################################################
## Perl modules
use strict;
use warnings;
use diagnostics;

use Getopt::Long;
use File::Basename;
use Data::Dumper;

##################################################
## Globals variables
##################################################

# Initializations
my $help = undef();
my $DEBUG = 0;
my %Parameters;
my @list_of_files = ();

##################################################
## Manage options
##################################################

# Get options from command line
GetOptions (	'help|h'				=> \$help,
			'output|out:s'		=> \$Parameters{'Main_output'},
			'list_name|name:s'	=> \$Parameters{'List_name'},
			'file_type|type:s'	=> \$Parameters{'File_type'},
			'list=s{,}'			=> \@list_of_files
		);

# Display help message if needed
if (defined($help)) {
	displayHelpMessage();
}

# Check parameters
checkParameters(\%Parameters, \@list_of_files);

##################################################
## Main code
##################################################


# Welcome message
print "###########################################################\n";
print "#         Welcome in File selector (Version $version)          #\n"; 
print "###########################################################\n\n";

# Verbose
print 'Selected file type: ' . $Parameters{'File_type'} . "\n\n";

print 'The list of input file given as argument contains ' . scalar(@list_of_files) . ' file(s):' . "\n\n";

# Write all complete path in the output file
open (PATH, '>' . $Parameters{'Main_output'}) or die ('Error: Cannot create/open file: ' . $Parameters{'Main_output'} . "\n");
foreach my $input_file (@list_of_files) {
	print PATH $input_file . "\n";
	print "\t-" . $input_file . "\n";
}
close (PATH);

print "\n" . 'Selected output file: ' . $Parameters{'Main_output'} . "\n\n";

print "###########################################################\n";
print "#                     End of execution                    #\n"; 
print "###########################################################\n";

##################################################
## Basic functions
##################################################

sub displayHelpMessage {

	print('################################################' . "\n");
	print('#        File selector  -  Help section        #' . "\n"); 
	print('################################################' . "\n\n");

	print "Usage example: File_selector.pl -out My_list_of_path.txt -name Ostrea_edulis_reads -type fastq -list file1.fastq file2.fastq...\n\n";
	
	print "Here is the list of authorized parameters :\n\n";

	print " -help => Display this help message.\n\n";
	
	print " -list multiple string => list of file (complete path) (mandatory).\n\n";

	print " -output/-out string => Path and name of the output file in txt format (optional).\n";
	print "\tIf not specified, a default name (derived from the selected list name) will be used.\n\n";
	
	print " -file_type/-type string => Type/Format of the selected files (optional).\n";
	print "\tIf not specified, a default type (txt) will be used.\n\n";

	print " -list_name/-name string => Name/Tag of the file selection (optional).\n";
	print "\tIf not specified, a default name (Unknown) will be used.\n\n";

	exit (1);
}

sub checkParameters {
	
	# Recovers parameters
	my ($Ref_to_parameters, $Ref_to_list) = @_;
	
	if (scalar(@{$Ref_to_list}) == 0) {
		print 'Error: the list of input file is empty...' . "\n\n";
		displayHelpMessage();
	}
	
	if (!defined($Ref_to_parameters->{'File_type'}) || $Ref_to_parameters->{'File_type'} eq "")  {
		print 'Warning: The <type> parameter has been omited ! Default type (txt) will be used...' . "\n\n";
		$Ref_to_parameters->{'File_type'} = 'txt';		
	}
	
	if (!defined($Ref_to_parameters->{'List_name'}) || $Ref_to_parameters->{'List_name'} eq "")  {
		print 'Warning: The <name> parameter has been omited ! Default name ("Unknown") will be used...' . "\n\n";
		$Ref_to_parameters->{'List_name'} = 'Unknown';		
	}
	
	if (!defined($Ref_to_parameters->{'Main_output'}))  {
		print 'Warning: The name of the output file has been omited ! A default name (derived from the selected list name) will be used !' . "\n\n";	
		$Ref_to_parameters->{'Main_output'} = 'File_selection_(' . $Ref_to_parameters->{'List_name'} . ').txt';
	}
}